@@ -2,10 +2,10 @@
 //! to load and inspect `Cargo.toml` metadata.
 //!
 //! See `Manifest::from_slice`.
+
 use std::fs;
 use std::io;
 use std::path::Path;
-use toml;
 
 #[macro_use]
 extern crate serde_derive;
@@ -32,7 +32,7 @@ pub use crate::error::Error;
 #[serde(rename_all = "kebab-case")]
 pub struct Manifest<Metadata = Value> {
     pub package: Option<Package<Metadata>>,
-    pub workspace: Option<Workspace>,
+    pub workspace: Option<Workspace<Metadata>>,
     #[serde(default, serialize_with = "toml::ser::tables_last")]
     pub dependencies: DepsSet,
     #[serde(default, serialize_with = "toml::ser::tables_last")]
@@ -70,7 +70,7 @@ pub struct Manifest<Metadata = Value> {
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize, Default)]
 #[serde(rename_all = "kebab-case")]
-pub struct Workspace {
+pub struct Workspace<Metadata = Value> {
     #[serde(default)]
     pub members: Vec<String>,
 
@@ -79,6 +79,8 @@ pub struct Workspace {
 
     #[serde(default)]
     pub exclude: Vec<String>,
+
+    pub metadata: Option<Metadata>,
 }
 
 fn default_true() -> bool {
@@ -378,7 +380,7 @@ impl Dependency {
     pub fn req(&self) -> &str {
         match *self {
             Dependency::Simple(ref v) => v,
-            Dependency::Detailed(ref d) => d.version.as_ref().map(|s| s.as_str()).unwrap_or("*"),
+            Dependency::Detailed(ref d) => d.version.as_deref().unwrap_or("*"),
         }
     }
 
@@ -398,14 +400,14 @@ impl Dependency {
     pub fn package(&self) -> Option<&str> {
         match *self {
             Dependency::Simple(_) => None,
-            Dependency::Detailed(ref d) => d.package.as_ref().map(|p| p.as_str()),
+            Dependency::Detailed(ref d) => d.package.as_deref(),
         }
     }
 
     // Git URL of this dependency, if any
     pub fn git(&self) -> Option<&str> {
         self.detail()
-            .and_then(|d| d.git.as_ref().map(|p| p.as_str()))
+            .and_then(|d| d.git.as_deref())
     }
 
     // `true` if it's an usual crates.io dependency,
@@ -470,6 +472,7 @@ pub struct Package<Metadata = Value> {
     pub documentation: Option<String>,
     /// This points to a file under the package root (relative to this `Cargo.toml`).
     /// implied if README.md, README.txt or README exists.
+    #[serde(default, deserialize_with = "readme_parser")]
     pub readme: Option<String>,
     #[serde(default)]
     pub keywords: Vec<String>,
@@ -518,7 +521,7 @@ impl PartialEq<Publish> for bool {
     fn eq(&self, p: &Publish) -> bool {
         match *p {
             Publish::Flag(flag) => flag == *self,
-            Publish::Registry(ref reg) => !reg.is_empty() == *self,
+            Publish::Registry(ref reg) => reg.is_empty() != *self,
         }
     }
 }
@@ -655,3 +658,43 @@ impl Default for Resolver {
         Self::V1
     }
 }
+
+fn readme_parser<'de, D>(deser: D) -> Result<Option<String>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    use serde::de;
+    use serde::de::Visitor;
+    use std::fmt;
+
+    struct ReadmeVisitor;
+    impl<'de> Visitor<'de> for ReadmeVisitor {
+        type Value = Option<String>;
+
+        fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+            write!(formatter, "readme property must be a stirng or bool")
+        }
+
+        fn visit_none<E: de::Error>(self) -> Result<Self::Value, E> {
+            Ok(None)
+        }
+
+        fn visit_unit<E: de::Error>(self) -> Result<Self::Value, E> {
+            Ok(None)
+        }
+
+        fn visit_bool<E: de::Error>(self, _: bool) -> Result<Self::Value, E> {
+            Ok(None) // unfortunately the return type can't capture the difference between none and false
+        }
+
+        fn visit_str<E: de::Error>(self, val: &str) -> Result<Self::Value, E> {
+            Ok(Some(val.to_string()))
+        }
+
+        fn visit_string<E: de::Error>(self, val: String) -> Result<Self::Value, E> {
+            Ok(Some(val))
+        }
+    }
+
+    deser.deserialize_any(ReadmeVisitor)
+}
